"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertProgramNode = exports.isProgramNode = exports.programNodeFromIdl = exports.programNode = void 0;
const shared_1 = require("../shared");
const AccountNode_1 = require("./AccountNode");
const DefinedTypeNode_1 = require("./DefinedTypeNode");
const ErrorNode_1 = require("./ErrorNode");
const InstructionNode_1 = require("./InstructionNode");
function programNode(input) {
    var _a, _b;
    return {
        kind: 'programNode',
        accounts: input.accounts,
        instructions: input.instructions,
        definedTypes: input.definedTypes,
        errors: input.errors,
        name: (0, shared_1.mainCase)(input.name),
        prefix: (0, shared_1.mainCase)((_a = input.prefix) !== null && _a !== void 0 ? _a : ''),
        publicKey: input.publicKey,
        version: input.version,
        origin: input.origin,
        internal: (_b = input.internal) !== null && _b !== void 0 ? _b : false,
    };
}
exports.programNode = programNode;
function programNodeFromIdl(idl) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j;
    const origin = (_a = idl.metadata) === null || _a === void 0 ? void 0 : _a.origin;
    const accounts = ((_b = idl.accounts) !== null && _b !== void 0 ? _b : []).map(AccountNode_1.accountNodeFromIdl);
    const instructions = ((_c = idl.instructions) !== null && _c !== void 0 ? _c : []).map((ix) => {
        var _a;
        return origin === 'anchor'
            ? (0, InstructionNode_1.instructionNodeFromIdl)({
                ...ix,
                defaultOptionalAccounts: (_a = ix.defaultOptionalAccounts) !== null && _a !== void 0 ? _a : true,
            })
            : (0, InstructionNode_1.instructionNodeFromIdl)(ix);
    });
    return programNode({
        accounts,
        instructions,
        definedTypes: ((_d = idl.types) !== null && _d !== void 0 ? _d : []).map(DefinedTypeNode_1.definedTypeNodeFromIdl),
        errors: ((_e = idl.errors) !== null && _e !== void 0 ? _e : []).map(ErrorNode_1.errorNodeFromIdl),
        name: (_f = idl.name) !== null && _f !== void 0 ? _f : '',
        publicKey: (_h = (_g = idl.metadata) === null || _g === void 0 ? void 0 : _g.address) !== null && _h !== void 0 ? _h : '',
        version: (_j = idl.version) !== null && _j !== void 0 ? _j : '',
        origin,
    });
}
exports.programNodeFromIdl = programNodeFromIdl;
function isProgramNode(node) {
    return !!node && node.kind === 'programNode';
}
exports.isProgramNode = isProgramNode;
function assertProgramNode(node) {
    var _a;
    if (!isProgramNode(node)) {
        throw new Error(`Expected programNode, got ${(_a = node === null || node === void 0 ? void 0 : node.kind) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertProgramNode = assertProgramNode;
//# sourceMappingURL=ProgramNode.js.map