"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertRootNode = exports.isRootNode = exports.getAllErrors = exports.getAllInstructions = exports.getAllDefinedTypes = exports.getAllAccounts = exports.rootNodeFromIdls = exports.rootNode = void 0;
const shared_1 = require("../shared");
const ProgramNode_1 = require("./ProgramNode");
function rootNode(programs) {
    return { kind: 'rootNode', programs };
}
exports.rootNode = rootNode;
function rootNodeFromIdls(idls) {
    const idlArray = Array.isArray(idls) ? idls : [idls];
    const programs = idlArray
        .map((idl) => (typeof idl === 'string' ? (0, shared_1.readJson)(idl) : idl))
        .map((idl) => (0, ProgramNode_1.programNodeFromIdl)(idl));
    return rootNode(programs);
}
exports.rootNodeFromIdls = rootNodeFromIdls;
function getAllAccounts(node) {
    return node.programs.flatMap((program) => program.accounts);
}
exports.getAllAccounts = getAllAccounts;
function getAllDefinedTypes(node) {
    return node.programs.flatMap((program) => program.definedTypes);
}
exports.getAllDefinedTypes = getAllDefinedTypes;
function getAllInstructions(node) {
    return node.programs.flatMap((program) => program.instructions);
}
exports.getAllInstructions = getAllInstructions;
function getAllErrors(node) {
    return node.programs.flatMap((program) => program.errors);
}
exports.getAllErrors = getAllErrors;
function isRootNode(node) {
    return !!node && node.kind === 'rootNode';
}
exports.isRootNode = isRootNode;
function assertRootNode(node) {
    var _a;
    if (!isRootNode(node)) {
        throw new Error(`Expected rootNode, got ${(_a = node === null || node === void 0 ? void 0 : node.kind) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertRootNode = assertRootNode;
//# sourceMappingURL=RootNode.js.map