"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertSetTypeNode = exports.isSetTypeNode = exports.setTypeNodeFromIdl = exports.setTypeNode = void 0;
const shared_1 = require("../shared");
const NumberTypeNode_1 = require("./NumberTypeNode");
const TypeNode_1 = require("./TypeNode");
function setTypeNode(child, options = {}) {
    var _a, _b;
    return {
        kind: 'setTypeNode',
        child,
        size: (_a = options.size) !== null && _a !== void 0 ? _a : (0, shared_1.prefixedSize)(),
        idlSet: (_b = options.idlSet) !== null && _b !== void 0 ? _b : 'hashSet',
    };
}
exports.setTypeNode = setTypeNode;
function setTypeNodeFromIdl(idl) {
    const child = 'hashSet' in idl ? idl.hashSet : idl.bTreeSet;
    let size;
    if (idl.size === 'remainder') {
        size = (0, shared_1.remainderSize)();
    }
    else if (typeof idl.size === 'number') {
        size = (0, shared_1.fixedSize)(idl.size);
    }
    else if (idl.size) {
        size = (0, shared_1.prefixedSize)((0, NumberTypeNode_1.numberTypeNode)(idl.size));
    }
    return setTypeNode((0, TypeNode_1.createTypeNodeFromIdl)(child), {
        size,
        idlSet: 'hashSet' in idl ? 'hashSet' : 'bTreeSet',
    });
}
exports.setTypeNodeFromIdl = setTypeNodeFromIdl;
function isSetTypeNode(node) {
    return !!node && node.kind === 'setTypeNode';
}
exports.isSetTypeNode = isSetTypeNode;
function assertSetTypeNode(node) {
    var _a;
    if (!isSetTypeNode(node)) {
        throw new Error(`Expected setTypeNode, got ${(_a = node === null || node === void 0 ? void 0 : node.kind) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertSetTypeNode = assertSetTypeNode;
//# sourceMappingURL=SetTypeNode.js.map