"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteFolder = exports.resolveTemplate = exports.createFile = exports.createDirectory = void 0;
const fs_1 = require("fs");
const nunjucks_1 = __importDefault(require("nunjucks"));
const shared_1 = require("../shared");
const createDirectory = (path) => {
    (0, fs_1.mkdirSync)(path, { recursive: true });
};
exports.createDirectory = createDirectory;
const createFile = (path, content) => {
    const directory = path.substring(0, path.lastIndexOf('/'));
    if (!(0, fs_1.existsSync)(directory)) {
        (0, exports.createDirectory)(directory);
    }
    (0, fs_1.writeFileSync)(path, content);
};
exports.createFile = createFile;
const resolveTemplate = (directory, file, context, options) => {
    const env = nunjucks_1.default.configure(directory, {
        trimBlocks: true,
        autoescape: false,
        ...options,
    });
    env.addFilter('pascalCase', shared_1.pascalCase);
    env.addFilter('camelCase', shared_1.camelCase);
    env.addFilter('snakeCase', shared_1.snakeCase);
    env.addFilter('kebabCase', shared_1.kebabCase);
    env.addFilter('titleCase', shared_1.titleCase);
    return env.render(file, context);
};
exports.resolveTemplate = resolveTemplate;
const deleteFolder = (path) => {
    if ((0, fs_1.existsSync)(path)) {
        (0, fs_1.rmSync)(path, { recursive: true });
    }
};
exports.deleteFolder = deleteFolder;
//# sourceMappingURL=utils.js.map