"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDefaultSeedsFromAccount = exports.dependsOnArg = exports.dependsOnAccount = exports.resolverDefault = exports.pdaDefault = exports.valueDefault = exports.argDefault = exports.accountBumpDefault = exports.accountDefault = exports.publicKeyDefault = exports.programDefault = exports.programIdDefault = exports.payerDefault = exports.identityDefault = void 0;
const nodes_1 = require("../nodes");
const utils_1 = require("./utils");
const identityDefault = () => ({
    kind: 'identity',
});
exports.identityDefault = identityDefault;
const payerDefault = () => ({
    kind: 'payer',
});
exports.payerDefault = payerDefault;
const programIdDefault = () => ({
    kind: 'programId',
});
exports.programIdDefault = programIdDefault;
const programDefault = (name, publicKey) => ({
    kind: 'program',
    program: { name: (0, utils_1.mainCase)(name), publicKey },
});
exports.programDefault = programDefault;
const publicKeyDefault = (publicKey) => ({
    kind: 'publicKey',
    publicKey,
});
exports.publicKeyDefault = publicKeyDefault;
const accountDefault = (account) => ({
    kind: 'account',
    name: (0, utils_1.mainCase)(account),
});
exports.accountDefault = accountDefault;
const accountBumpDefault = (account) => ({
    kind: 'accountBump',
    name: (0, utils_1.mainCase)(account),
});
exports.accountBumpDefault = accountBumpDefault;
const argDefault = (arg) => ({
    kind: 'arg',
    name: (0, utils_1.mainCase)(arg),
});
exports.argDefault = argDefault;
const valueDefault = (value) => ({ kind: 'value', value });
exports.valueDefault = valueDefault;
const pdaDefault = (pdaAccount, options = {}) => {
    var _a, _b;
    return ({
        kind: 'pda',
        pdaAccount: (0, utils_1.mainCase)(pdaAccount),
        importFrom: (_a = options.importFrom) !== null && _a !== void 0 ? _a : 'generated',
        seeds: (_b = options.seeds) !== null && _b !== void 0 ? _b : {},
    });
};
exports.pdaDefault = pdaDefault;
const resolverDefault = (name, dependsOn, options = {}) => {
    var _a;
    return ({
        kind: 'resolver',
        name: (0, utils_1.mainCase)(name),
        importFrom: (_a = options.importFrom) !== null && _a !== void 0 ? _a : 'hooked',
        dependsOn,
        resolvedIsSigner: options.resolvedIsSigner,
        resolvedIsOptional: options.resolvedIsOptional,
    });
};
exports.resolverDefault = resolverDefault;
const dependsOnAccount = (account) => ({
    kind: 'account',
    name: (0, utils_1.mainCase)(account),
});
exports.dependsOnAccount = dependsOnAccount;
const dependsOnArg = (arg) => ({
    kind: 'arg',
    name: (0, utils_1.mainCase)(arg),
});
exports.dependsOnArg = dependsOnArg;
const getDefaultSeedsFromAccount = (node) => node.seeds.reduce((acc, seed) => {
    if (seed.kind !== 'variable')
        return acc;
    if ((0, nodes_1.isPublicKeyTypeNode)(seed.type)) {
        acc[seed.name] = { kind: 'account', name: seed.name };
    }
    else {
        acc[seed.name] = { kind: 'arg', name: seed.name };
    }
    return acc;
}, {});
exports.getDefaultSeedsFromAccount = getDefaultSeedsFromAccount;
//# sourceMappingURL=InstructionDefault.js.map