"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UnwrapDefinedTypesVisitor = void 0;
const nodes = __importStar(require("../../nodes"));
const shared_1 = require("../../shared");
const BaseNodeVisitor_1 = require("../BaseNodeVisitor");
const Visitor_1 = require("../Visitor");
class UnwrapDefinedTypesVisitor extends BaseNodeVisitor_1.BaseNodeVisitor {
    constructor(typesToInline = '*') {
        super();
        this.availableDefinedTypes = new Map();
        this.typesToInline =
            typesToInline === '*' ? '*' : typesToInline.map(shared_1.mainCase);
    }
    visitRoot(root) {
        nodes.getAllDefinedTypes(root).forEach((definedType) => {
            this.availableDefinedTypes.set(definedType.name, definedType);
        });
        return super.visitRoot(root);
    }
    visitProgram(program) {
        return nodes.programNode({
            ...program,
            accounts: program.accounts
                .map((account) => (0, Visitor_1.visit)(account, this))
                .filter(nodes.assertNodeFilter(nodes.assertAccountNode)),
            instructions: program.instructions
                .map((instruction) => (0, Visitor_1.visit)(instruction, this))
                .filter(nodes.assertNodeFilter(nodes.assertInstructionNode)),
            definedTypes: program.definedTypes
                .filter((definedType) => !this.shouldInline(definedType.name))
                .map((type) => (0, Visitor_1.visit)(type, this))
                .filter(nodes.assertNodeFilter(nodes.assertDefinedTypeNode)),
        });
    }
    visitLinkType(linkType) {
        if (!this.shouldInline(linkType.name) ||
            linkType.importFrom !== 'generated') {
            return linkType;
        }
        const definedType = this.availableDefinedTypes.get(linkType.name);
        if (definedType === undefined) {
            throw new Error(`Trying to inline missing defined type [${linkType.name}]. ` +
                `Ensure this visitor starts from the root node to access all defined types.`);
        }
        return (0, Visitor_1.visit)(definedType.data, this);
    }
    shouldInline(definedType) {
        return (this.typesToInline === '*' || this.typesToInline.includes(definedType));
    }
}
exports.UnwrapDefinedTypesVisitor = UnwrapDefinedTypesVisitor;
//# sourceMappingURL=UnwrapDefinedTypesVisitor.js.map