"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UpdateInstructionsVisitor = void 0;
const nodes = __importStar(require("../../nodes"));
const shared_1 = require("../../shared");
const TransformNodesVisitor_1 = require("./TransformNodesVisitor");
class UpdateInstructionsVisitor extends TransformNodesVisitor_1.TransformNodesVisitor {
    constructor(map) {
        const transforms = Object.entries(map).map(([selector, updates]) => {
            const selectorStack = selector.split('.');
            const name = selectorStack.pop();
            return {
                selector: { kind: 'instructionNode', stack: selectorStack, name },
                transformer: (node, stack, program) => {
                    var _a;
                    nodes.assertInstructionNode(node);
                    if (typeof updates === 'function') {
                        return updates(node, stack, program);
                    }
                    if ('delete' in updates) {
                        return null;
                    }
                    const { accounts: accountUpdates, args: argsUpdates, ...metadataUpdates } = updates;
                    const newName = (0, shared_1.mainCase)((_a = updates.name) !== null && _a !== void 0 ? _a : node.name);
                    const { newDataArgs, newExtraArgs, newArgDefaults } = this.handleInstructionArgs(node, newName, argsUpdates !== null && argsUpdates !== void 0 ? argsUpdates : {});
                    const newAccounts = node.accounts.map((account) => this.handleInstructionAccount(account, accountUpdates !== null && accountUpdates !== void 0 ? accountUpdates : {}));
                    return nodes.instructionNode({
                        ...node,
                        ...metadataUpdates,
                        argDefaults: newArgDefaults,
                        accounts: newAccounts,
                        dataArgs: newDataArgs,
                        extraArgs: newExtraArgs,
                    });
                },
            };
        });
        super(transforms);
        this.map = map;
        this.allAccounts = new Map();
    }
    visitRoot(root) {
        nodes.getAllAccounts(root).forEach((account) => {
            this.allAccounts.set(account.name, account);
        });
        return super.visitRoot(root);
    }
    handleInstructionAccount(account, accountUpdates) {
        const accountUpdate = accountUpdates === null || accountUpdates === void 0 ? void 0 : accountUpdates[account.name];
        if (!accountUpdate)
            return account;
        const { defaultsTo, ...acountWithoutDefault } = {
            ...account,
            ...accountUpdate,
        };
        if (defaultsTo === null) {
            return nodes.instructionAccountNode(acountWithoutDefault);
        }
        if ((defaultsTo === null || defaultsTo === void 0 ? void 0 : defaultsTo.kind) === 'pda') {
            const pdaAccount = (0, shared_1.mainCase)(defaultsTo.pdaAccount);
            const foundAccount = this.allAccounts.get(pdaAccount);
            return {
                ...acountWithoutDefault,
                defaultsTo: {
                    ...defaultsTo,
                    seeds: {
                        ...(foundAccount ? (0, shared_1.getDefaultSeedsFromAccount)(foundAccount) : {}),
                        ...defaultsTo.seeds,
                    },
                },
            };
        }
        return nodes.instructionAccountNode({
            ...acountWithoutDefault,
            defaultsTo,
        });
    }
    handleInstructionArgs(instruction, newInstructionName, argUpdates) {
        const usedArgs = new Set();
        const newDataArgs = nodes.instructionDataArgsNode({
            ...instruction.dataArgs,
            name: `${newInstructionName}InstructionData`,
            struct: nodes.structTypeNode(instruction.dataArgs.struct.fields.map((field) => {
                var _a, _b, _c;
                const argUpdate = argUpdates[field.name];
                if (!argUpdate)
                    return field;
                usedArgs.add(field.name);
                return nodes.structFieldTypeNode({
                    ...field,
                    child: (_a = argUpdate.type) !== null && _a !== void 0 ? _a : field.child,
                    name: (_b = argUpdate.name) !== null && _b !== void 0 ? _b : field.name,
                    docs: (_c = argUpdate.docs) !== null && _c !== void 0 ? _c : field.docs,
                });
            })),
        });
        const updatedExtraFields = instruction.extraArgs.struct.fields.map((field) => {
            var _a, _b, _c;
            if (usedArgs.has(field.name))
                return field;
            const argUpdate = argUpdates[field.name];
            if (!argUpdate)
                return field;
            usedArgs.add(field.name);
            return nodes.structFieldTypeNode({
                ...field,
                child: (_a = argUpdate.type) !== null && _a !== void 0 ? _a : field.child,
                name: (_b = argUpdate.name) !== null && _b !== void 0 ? _b : field.name,
                docs: (_c = argUpdate.docs) !== null && _c !== void 0 ? _c : field.docs,
            });
        });
        const newExtraFields = Object.entries(argUpdates)
            .filter(([argName]) => !usedArgs.has(argName))
            .map(([argName, argUpdate]) => {
            var _a, _b, _c;
            const child = (_a = argUpdate.type) !== null && _a !== void 0 ? _a : null;
            nodes.assertTypeNode(child);
            return nodes.structFieldTypeNode({
                name: (_b = argUpdate.name) !== null && _b !== void 0 ? _b : argName,
                child,
                docs: (_c = argUpdate.docs) !== null && _c !== void 0 ? _c : [],
            });
        });
        const newExtraArgs = nodes.instructionExtraArgsNode({
            ...instruction.extraArgs,
            name: `${newInstructionName}InstructionExtra`,
            struct: nodes.structTypeNode([...updatedExtraFields, ...newExtraFields]),
        });
        const newArgDefaults = instruction.argDefaults;
        Object.entries(argUpdates).forEach(([argName, argUpdate]) => {
            if ((argUpdate === null || argUpdate === void 0 ? void 0 : argUpdate.defaultsTo) === undefined)
                return;
            if (argUpdate.defaultsTo === null) {
                delete newArgDefaults[argName];
            }
            else {
                newArgDefaults[argName] = argUpdate.defaultsTo;
            }
        });
        return { newDataArgs, newExtraArgs, newArgDefaults };
    }
}
exports.UpdateInstructionsVisitor = UpdateInstructionsVisitor;
//# sourceMappingURL=UpdateInstructionsVisitor.js.map