import type { IdlInstruction } from '../idl';
import { BytesCreatedOnChain, InstructionArgDefault, PartialExcept } from '../shared';
import { InstructionAccountNode } from './InstructionAccountNode';
import { InstructionDataArgsNode } from './InstructionDataArgsNode';
import { InstructionExtraArgsNode } from './InstructionExtraArgsNode';
import type { Node } from './Node';
import { ProgramNode } from './ProgramNode';
import { RootNode } from './RootNode';
export type InstructionNode = {
    readonly __instructionNode: unique symbol;
    readonly kind: 'instructionNode';
    readonly name: string;
    readonly accounts: InstructionAccountNode[];
    readonly dataArgs: InstructionDataArgsNode;
    readonly extraArgs: InstructionExtraArgsNode;
    readonly subInstructions: InstructionNode[];
    readonly idlName: string;
    readonly docs: string[];
    readonly internal: boolean;
    readonly bytesCreatedOnChain?: BytesCreatedOnChain;
    readonly argDefaults: Record<string, InstructionArgDefault>;
};
export type InstructionNodeInput = Omit<PartialExcept<InstructionNode, 'name' | 'accounts' | 'dataArgs'>, '__instructionNode' | 'kind'>;
export declare function instructionNode(input: InstructionNodeInput): InstructionNode;
export declare function instructionNodeFromIdl(idl: Partial<IdlInstruction>): InstructionNode;
export declare function getAllInstructionsWithSubs(node: ProgramNode | RootNode | InstructionNode, leavesOnly?: boolean): InstructionNode[];
export declare function isInstructionNode(node: Node | null): node is InstructionNode;
export declare function assertInstructionNode(node: Node | null): asserts node is InstructionNode;
