import { ImportFrom } from '../shared';
export type ValueNode = ScalarValueNode | PublicKeyValueNode | ListValueNode | TupleValueNode | SetValueNode | MapValueNode | OptionValueNode | StructValueNode | EnumValueNode;
export type ScalarValueNode = {
    kind: 'number';
    value: number;
} | {
    kind: 'boolean';
    value: boolean;
} | {
    kind: 'string';
    value: string;
};
export declare const vScalar: (scalar: number | boolean | string) => ScalarValueNode;
export type PublicKeyValueNode = {
    kind: 'publicKey';
    value: string;
};
export declare const vPublicKey: (value: string) => PublicKeyValueNode;
export type ListValueNode = {
    kind: 'list';
    values: ValueNode[];
};
export declare const vList: (values: ValueNode[]) => ListValueNode;
export type TupleValueNode = {
    kind: 'tuple';
    values: ValueNode[];
};
export declare const vTuple: (values: ValueNode[]) => TupleValueNode;
export type SetValueNode = {
    kind: 'set';
    values: ValueNode[];
};
export declare const vSet: (values: ValueNode[]) => SetValueNode;
export type MapValueNode = {
    kind: 'map';
    values: [ValueNode, ValueNode][];
};
export declare const vMap: (values: [ValueNode, ValueNode][]) => MapValueNode;
export type OptionValueNode = {
    kind: 'optionNone';
} | {
    kind: 'optionSome';
    value: ValueNode;
};
export declare const vNone: () => OptionValueNode;
export declare const vSome: (value: ValueNode) => OptionValueNode;
export type StructValueNode = {
    kind: 'struct';
    values: Record<string, ValueNode>;
};
export declare const vStruct: (values: Record<string, ValueNode>) => StructValueNode;
export type EnumValueNode = {
    kind: 'enum';
    enumType: string;
    variant: string;
    value: StructValueNode | TupleValueNode | 'empty' | 'scalar';
    importFrom: ImportFrom | null;
};
export declare const vEnum: (enumType: string, variant: string, value?: StructValueNode | TupleValueNode | 'empty' | 'scalar', importFrom?: ImportFrom | null) => EnumValueNode;
