import { InstructionArgDefault, InstructionDependency } from '../../shared';
import type * as nodes from '../../nodes';
import { BaseThrowVisitor } from '../BaseThrowVisitor';
type InstructionInput = InstructionArg | InstructionAccount;
type InstructionArg = {
    kind: 'arg';
    name: string;
    defaultsTo: InstructionArgDefault;
};
type InstructionAccount = {
    kind: 'account';
} & Omit<nodes.InstructionAccountNode, 'kind'>;
export type ResolvedInstructionInput = ResolvedInstructionAccount | ResolvedInstructionArg;
export type ResolvedInstructionAccount = InstructionAccount & {
    isPda: boolean;
    dependsOn: InstructionDependency[];
    resolvedIsSigner: boolean | 'either';
    resolvedIsOptional: boolean;
};
export type ResolvedInstructionArg = InstructionArg & {
    dependsOn: InstructionDependency[];
};
export declare class GetResolvedInstructionInputsVisitor extends BaseThrowVisitor<ResolvedInstructionInput[]> {
    protected stack: InstructionInput[];
    protected resolved: ResolvedInstructionInput[];
    protected visitedAccounts: Map<string, ResolvedInstructionAccount>;
    protected visitedArgs: Map<string, ResolvedInstructionArg>;
    protected error: string | null;
    getError(): string | null;
    visitInstruction(instruction: nodes.InstructionNode): ResolvedInstructionInput[];
    resolveInstructionInput(instruction: nodes.InstructionNode, input: InstructionInput): void;
    resolveInstructionAccount(instruction: nodes.InstructionNode, account: InstructionAccount): ResolvedInstructionAccount;
    resolveInstructionArg(instruction: nodes.InstructionNode, arg: InstructionArg & {
        kind: 'arg';
    }): ResolvedInstructionArg;
    resolveInstructionDependencies(instruction: nodes.InstructionNode, parent: InstructionInput, dependencies: InstructionDependency[]): void;
    getInstructionDependencies(input: InstructionInput): InstructionDependency[];
}
export {};
