import * as nodes from '../../nodes';
import { InstructionAccountDefault, InstructionArgDefault } from '../../shared';
import { NodeTransformer, TransformNodesVisitor } from './TransformNodesVisitor';
export type InstructionUpdates = NodeTransformer<nodes.InstructionNode> | {
    delete: true;
} | (InstructionMetadataUpdates & {
    accounts?: InstructionAccountUpdates;
    args?: InstructionArgUpdates;
});
export type InstructionMetadataUpdates = Partial<Omit<nodes.InstructionNodeInput, 'accounts' | 'dataArgs' | 'extraArgs' | 'subInstructions' | 'argDefaults'>>;
export type InstructionAccountUpdates = Record<string, Partial<Omit<nodes.InstructionAccountNodeInput, 'defaultsTo'>> & {
    defaultsTo?: InstructionAccountDefault | null;
}>;
export type InstructionArgUpdates = Record<string, {
    name?: string;
    docs?: string[];
    type?: nodes.TypeNode;
    defaultsTo?: InstructionArgDefault | null;
}>;
export declare class UpdateInstructionsVisitor extends TransformNodesVisitor {
    readonly map: Record<string, InstructionUpdates>;
    protected allAccounts: Map<string, nodes.AccountNode>;
    constructor(map: Record<string, InstructionUpdates>);
    visitRoot(root: nodes.RootNode): nodes.Node | null;
    handleInstructionAccount(account: nodes.InstructionAccountNode, accountUpdates: InstructionAccountUpdates): nodes.InstructionAccountNode;
    handleInstructionArgs(instruction: nodes.InstructionNode, newInstructionName: string, argUpdates: InstructionArgUpdates): {
        newDataArgs: nodes.InstructionDataArgsNode;
        newExtraArgs: nodes.InstructionExtraArgsNode;
        newArgDefaults: Record<string, InstructionArgDefault>;
    };
}
